@@ -8,28 +8,38 @@
 ### rowsum() / colsum()
 ###
 
-colsum <- function(x, group, reorder=TRUE, na.rm=FALSE, ...)
-{
-    t(rowsum(t(x), group, reorder=reorder, na.rm=na.rm, ...))
-}
-
-.compute_rowsum_for_block <- function(x, grid, i, j, group, na.rm=FALSE)
+.compute_rowsum_for_block <- function(x, grid, i, j, x_is_sparse,
+                                      group, na.rm=FALSE)
 {
     viewport <- grid[[i, j]]
-    block <- read_block(x, viewport)
+    if (x_is_sparse) {
+        block <- read_sparse_block(x, viewport)
+        ## Go from SparseArraySeed to dgCMatrix.
+        block <- as(block, "dgCMatrix")
+    } else {
+        block <- read_block(x, viewport)
+    }
     group2 <- extractROWS(group, ranges(viewport)[1L])
     rowsum(block, group2, reorder=FALSE, na.rm=na.rm)
 }
-.compute_colsum_for_block <- function(x, grid, i, j, group, na.rm=FALSE)
+.compute_colsum_for_block <- function(x, grid, i, j, x_is_sparse,
+                                      group, na.rm=FALSE)
 {
     viewport <- grid[[i, j]]
-    block <- read_block(x, viewport)
+    if (x_is_sparse) {
+        block <- read_sparse_block(x, viewport)
+        ## Go from SparseArraySeed to dgCMatrix.
+        block <- as(block, "dgCMatrix")
+    } else {
+        block <- read_block(x, viewport)
+    }
     group2 <- extractROWS(group, ranges(viewport)[2L])
     colsum(block, group2, reorder=FALSE, na.rm=na.rm)
 }
 
-.compute_rowsum_for_grid_col <- function(x, grid, j, group, ugroup,
-                                         na.rm=FALSE, verbose=FALSE)
+.compute_rowsum_for_grid_col <- function(x, grid, j, x_is_sparse,
+                                         group, ugroup, na.rm=FALSE,
+                                         verbose=FALSE)
 {
     grid_nrow <- nrow(grid)
     grid_ncol <- ncol(grid)
@@ -40,7 +50,7 @@ colsum <- function(x, group, reorder=TRUE, na.rm=FALSE, ...)
             message("Processing block [[", i, "/", grid_nrow, ", ",
                                            j, "/", grid_ncol, "]] ... ",
                     appendLF=FALSE)
-        block_ans <- .compute_rowsum_for_block(x, grid, i, j,
+        block_ans <- .compute_rowsum_for_block(x, grid, i, j, x_is_sparse,
                                                group, na.rm=na.rm)
         m <- match(rownames(block_ans), ugroup)
         ans[m, ] <- ans[m, ] + block_ans
@@ -49,8 +59,9 @@ colsum <- function(x, group, reorder=TRUE, na.rm=FALSE, ...)
     }
     ans
 }
-.compute_colsum_for_grid_row <- function(x, grid, i, group, ugroup,
-                                         na.rm=FALSE, verbose=FALSE)
+.compute_colsum_for_grid_row <- function(x, grid, i, x_is_sparse,
+                                         group, ugroup, na.rm=FALSE,
+                                         verbose=FALSE)
 {
     grid_nrow <- nrow(grid)
     grid_ncol <- ncol(grid)
@@ -61,7 +72,7 @@ colsum <- function(x, group, reorder=TRUE, na.rm=FALSE, ...)
             message("Processing block [[", i, "/", grid_nrow, ", ",
                                            j, "/", grid_ncol, "]] ... ",
                     appendLF=FALSE)
-        block_ans <- .compute_colsum_for_block(x, grid, i, j,
+        block_ans <- .compute_colsum_for_block(x, grid, i, j, x_is_sparse,
                                                group, na.rm=na.rm)
         m <- match(colnames(block_ans), ugroup)
         ans[ , m] <- ans[ , m] + block_ans
@@ -71,69 +82,51 @@ colsum <- function(x, group, reorder=TRUE, na.rm=FALSE, ...)
     ans
 }
 
-.compute_ugroup <- function(group, expected_group_len, reorder=TRUE)
-{
-    if (!(is.vector(group) || is.factor(group)))
-        stop(wmsg("'group' must be a vector or factor"))
-    if (length(group) != expected_group_len)
-        stop(wmsg("incorrect length for 'group'"))
-    if (!isTRUEorFALSE(reorder))
-        stop(wmsg("'reorder' must be TRUE or FALSE"))
-    ## Taken from base::rowsum.default().
-    ugroup <- unique(group)
-    if (anyNA(ugroup))
-        warning(wmsg("missing values for 'group'"))
-    if (reorder)
-        ugroup <- sort(ugroup, na.last=TRUE, method="quick")
-    as.character(ugroup)
-}
-
 .BLOCK_rowsum <- function(x, group, reorder=TRUE, na.rm=FALSE, grid=NULL)
 {
-    ugroup <- .compute_ugroup(group, nrow(x), reorder)
+    ugroup <- as.character(compute_ugroup(group, nrow(x), reorder))
     if (!isTRUEorFALSE(na.rm))
         stop(wmsg("'na.rm' must be TRUE or FALSE"))
     grid <- normarg_grid(grid, x)
+    x_is_sparse <- is_sparse(x)
 
     ## Outer loop on the grid columns. Parallelized.
-    block_results <- bplapply(seq_len(ncol(grid)),
+    block_results <- bplapply2(seq_len(ncol(grid)),
         function(j) {
-            .compute_rowsum_for_grid_col(x, grid, j, group, ugroup,
-                                         na.rm=na.rm,
+            .compute_rowsum_for_grid_col(x, grid, j, x_is_sparse,
+                                         group, ugroup, na.rm=na.rm,
                                          verbose=get_verbose_block_processing())
         },
         BPPARAM=getAutoBPPARAM()
     )
 
     ans <- do.call(cbind, block_results)
-    dimnames(ans) <- list(as.character(ugroup), colnames(x))
+    dimnames(ans) <- list(ugroup, colnames(x))
     ans
 }
 .BLOCK_colsum <- function(x, group, reorder=TRUE, na.rm=FALSE, grid=NULL)
 {
-    ugroup <- .compute_ugroup(group, ncol(x), reorder)
+    ugroup <- as.character(compute_ugroup(group, ncol(x), reorder))
     if (!isTRUEorFALSE(na.rm))
         stop(wmsg("'na.rm' must be TRUE or FALSE"))
     grid <- normarg_grid(grid, x)
+    x_is_sparse <- is_sparse(x)
 
     ## Outer loop on the grid rows. Parallelized.
-    block_results <- bplapply(seq_len(nrow(grid)),
+    block_results <- bplapply2(seq_len(nrow(grid)),
         function(i) {
-            .compute_colsum_for_grid_row(x, grid, i, group, ugroup,
-                                         na.rm=na.rm,
+            .compute_colsum_for_grid_row(x, grid, i, x_is_sparse,
+                                         group, ugroup, na.rm=na.rm,
                                          verbose=get_verbose_block_processing())
         },
         BPPARAM=getAutoBPPARAM()
     )
 
     ans <- do.call(rbind, block_results)
-    dimnames(ans) <- list(rownames(x), as.character(ugroup))
+    dimnames(ans) <- list(rownames(x), ugroup)
     ans
 }
 
-setGeneric("rowsum", signature="x")
-setGeneric("colsum", signature="x")
-
 ### S3/S4 combo for rowsum.DelayedMatrix
 rowsum.DelayedMatrix <- function(x, group, reorder=TRUE, ...)
     .BLOCK_rowsum(x, group, reorder=reorder, ...)
@@ -166,14 +159,14 @@ setMethod("colsum", "DelayedMatrix", .BLOCK_colsum)
     ans_spacings <- c(ans_dim[[1L]], ncol(y_grid[[1L]]))
     ans_grid <- RegularArrayGrid(ans_dim, ans_spacings)  # parallel to 'y_grid'
     nblock <- length(y_grid)  # same as 'length(ans_grid)'
-    for (b in seq_len(nblock)) {
+    for (bid in seq_len(nblock)) {
         if (get_verbose_block_processing())
-            message("Processing block ", b, "/", nblock, " ... ",
+            message("Processing block ", bid, "/", nblock, " ... ",
                     appendLF=FALSE)
-        y_viewport <- y_grid[[b]]
+        y_viewport <- y_grid[[bid]]
         block <- read_block(y, y_viewport)
         block_ans <- x %*% block
-        write_block(sink, ans_grid[[b]], block_ans)
+        write_block(sink, ans_grid[[bid]], block_ans)
         if (get_verbose_block_processing())
             message("OK")
     }
@@ -490,14 +483,14 @@ setMethod("%*%", c("DelayedMatrix", "DelayedMatrix"), .BLOCK_matrix_mult)
 
 .single_grid_iterate <- function(grid) {
     force(grid)
-    b <- 0L
+    bid <- 0L
 
     # Do NOT put read_block in here, as this will waste time realizing from file in the single worker.
     # TODO: subset in-memory matrices to avoid serializing them in their entirety to the workers.
     function () {
-        if (b==length(grid)) return(NULL)
-        b <<- b + 1L
-        grid[[b]]
+        if (bid == length(grid)) return(NULL)
+        bid <<- bid + 1L
+        grid[[bid]]
     }
 }
 
@@ -514,12 +507,12 @@ setMethod("%*%", c("DelayedMatrix", "DelayedMatrix"), .BLOCK_matrix_mult)
 .dual_grid_iterate <- function(grid.x, grid.y) {
     force(grid.x)
     force(grid.y)
-    b <- 0L
+    bid <- 0L
 
     function () {
-        if (b==length(grid.x)) return(NULL)
-        b <<- b + 1L
-        list(x=grid.x[[b]], y=grid.y[[b]])
+        if (bid == length(grid.x)) return(NULL)
+        bid <<- bid + 1L
+        list(x=grid.x[[bid]], y=grid.y[[bid]])
     }
 }
 
@@ -533,17 +526,27 @@ setMethod("%*%", c("DelayedMatrix", "DelayedMatrix"), .BLOCK_matrix_mult)
 # Controller function that split jobs for a multiplication function "MULT".
 # This accommodates %*%, crossprod and tcrossprod for two arguments.
 {
-    scheme_out <- .dispatch_mult(x, y, bpnworkers(BPPARAM), transposed.x=transposed.x, transposed.y=transposed.y)
+    if (!requireNamespace("BiocParallel", quietly=TRUE))
+        stop(wmsg("Couldn't load the BiocParallel package. Please ",
+                  "install the BiocParallel package and try again."))
+    if (is.null(BPPARAM))
+        BPPARAM <- BiocParallel::SerialParam()
+    nworkers <- BiocParallel::bpnworkers(BPPARAM)
+    scheme_out <- .dispatch_mult(x, y, nworkers, transposed.x=transposed.x, transposed.y=transposed.y)
     chosen_scheme <- scheme_out$choice
     chosen_grids <- scheme_out$grid
 
     old <- getAutoBPPARAM()
     on.exit(setAutoBPPARAM(old))
-    setAutoBPPARAM(SerialParam()) # avoid re-parallelizing in further calls to 'MULT'.
+    ## Avoid re-parallelizing in further calls to 'MULT'.
+    setAutoBPPARAM(BiocParallel::SerialParam())
 
     if (chosen_scheme=="common") {
-        ans <- bpiterate(.dual_grid_iterate(chosen_grids$x, chosen_grids$y), FUN=.both_mult,
-            x=x, y=y, MULT=MULT, BPPARAM=BPPARAM, REDUCE=function(x, y) x + y)
+        ITER <- .dual_grid_iterate(chosen_grids$x, chosen_grids$y)
+        ans <- BiocParallel::bpiterate(ITER,
+                                       FUN=.both_mult, x=x, y=y, MULT=MULT,
+                                       BPPARAM=BPPARAM,
+                                       REDUCE=function(x, y) x + y)
 
     } else {
         # Switch to iteration over the other argument if the chosen one is
@@ -557,12 +560,16 @@ setMethod("%*%", c("DelayedMatrix", "DelayedMatrix"), .BLOCK_matrix_mult)
         }
 
         if (chosen_scheme=="x") {
-            out <- bpiterate(.single_grid_iterate(chosen_grids$x), FUN=.left_mult,
-                x=x, y=y, MULT=MULT, BPPARAM=BPPARAM)
+            ITER <- .single_grid_iterate(chosen_grids$x)
+            out <- BiocParallel::bpiterate(ITER,
+                                           FUN=.left_mult, x=x, y=y, MULT=MULT,
+                                           BPPARAM=BPPARAM)
             ans <- do.call(rbind, out)
         } else if (chosen_scheme=="y") {
-           out <- bpiterate(.single_grid_iterate(chosen_grids$y), FUN=.right_mult,
-                x=x, y=y, MULT=MULT, BPPARAM=BPPARAM)
+           ITER <- .single_grid_iterate(chosen_grids$y)
+           out <- BiocParallel::bpiterate(ITER,
+                                          FUN=.right_mult, x=x, y=y, MULT=MULT,
+                                          BPPARAM=BPPARAM)
            ans <- do.call(cbind, out)
         }
     }
@@ -613,22 +620,34 @@ setMethod("tcrossprod", c("DelayedMatrix", "DelayedMatrix"), function(x, y) .sup
 # Controller function that split jobs for a multiplication function "MULT".
 # This accommodates crossprod and tcrossprod for single arguments.
 {
-    scheme_out <- .dispatch_self(x, bpnworkers(BPPARAM), transposed=transposed)
+    if (!requireNamespace("BiocParallel", quietly=TRUE))
+        stop(wmsg("Couldn't load the BiocParallel package. Please ",
+                  "install the BiocParallel package and try again."))
+    if (is.null(BPPARAM))
+        BPPARAM <- BiocParallel::SerialParam()
+    nworkers <- BiocParallel::bpnworkers(BPPARAM)
+    scheme_out <- .dispatch_self(x, nworkers, transposed=transposed)
     chosen_scheme <- scheme_out$choice
     chosen_grids <- scheme_out$grid
 
     old <- getAutoBPPARAM()
     on.exit(setAutoBPPARAM(old))
-    setAutoBPPARAM(SerialParam()) # avoid re-parallelizing in further calls to 'MULT'.
+    ## Avoid re-parallelizing in further calls to 'MULT'.
+    setAutoBPPARAM(BiocParallel::SerialParam())
 
     if (chosen_scheme=="single") {
-        out <- bpiterate(.single_grid_iterate(chosen_grids), FUN=.left_mult,
-            x=x, y=x, MULT=MULT, BPPARAM=SerialParam())
+        ITER <- .single_grid_iterate(chosen_grids)
+        out <- BiocParallel::bpiterate(ITER,
+                                       FUN=.left_mult, x=x, y=x, MULT=MULT,
+                                       BPPARAM=BiocParallel::SerialParam())
         ans <- do.call(rbind, out)
 
     } else if (chosen_scheme=="both") {
-        ans <- bpiterate(.single_grid_iterate(chosen_grids), FUN=.solo_mult,
-            x=x, MULT=MULT, BPPARAM=BPPARAM, REDUCE=function(x, y) x+y)
+        ITER <- .single_grid_iterate(chosen_grids)
+        ans <- BiocParallel::bpiterate(ITER,
+                                       FUN=.solo_mult, x=x, MULT=MULT,
+                                       BPPARAM=BPPARAM,
+                                       REDUCE=function(x, y) x+y)
 
     }
 
